@@ -10,14 +10,20 @@ db.search("[?name=='Gregor']")
 [{'name': 'Gregor', 'age': 111}]
 
 """
-import oyaml as yaml
+# pylint: disable=C0103,W0107,W0702
 import os
-import jmespath
 
+import jmespath
+import oyaml as yaml
+from collections.abc import MutableMapping
+from contextlib import suppress
 
 class YamlDB:
+    """
+    The YamlBD class uses a file based yaml file as its database backend.
+    """
 
-    def __init__(self, data=None, filename=None, backend=":file:"):
+    def __init__(self, *, data=None, filename="yamldb.yml", backend=":file:"):
         """
         Initialized=s the data base, if data is not None it is
         used to initialize the DB.
@@ -27,11 +33,24 @@ class YamlDB:
         :param filename:
         :type filename:
         """
+        if backend not in [":file:", ":memory:"]:
+            raise ValueError("bakend must be :file: or :memory:")
         self.backend = backend
         self.filename = filename
-        if data is not None:
+
+        if os.path.exists(filename) and data is not None:
             self.data = data
-        self.flush()
+            self.save(filename=filename)
+        elif os.path.exists(filename) and data is None:
+            self.load(filename=filename)
+        elif not os.path.exists(filename) and data is not None:
+            self.data = data
+            self.save(filename=filename)
+        elif not os.path.exists(filename) and data is None:
+            self.data = {}
+            self.save(filename=filename)
+        else:
+            raise ValueError("Load failed")
 
     def __iter__(self):
         """
@@ -53,7 +72,7 @@ class YamlDB:
         """
         found = True
         try:
-            self[key]
+            self.__getitem__(key)
         except:
             found = False
         return found
@@ -66,6 +85,9 @@ class YamlDB:
         self.data.clear()
         self.flush()
 
+    def keys(self):
+        return self.data.keys()
+
     def load(self, filename=None):
         """
         Loads the data from the specified filename
@@ -85,6 +107,26 @@ class YamlDB:
             with open(name, 'rb') as dbfile:
                 self.data = yaml.safe_load(dbfile) or dict()
 
+    def update(self, filename=None):
+        """
+        Inserts the data from the specified filename
+
+        :param filename:
+        :type filename:
+        :return:
+        :rtype:
+        """
+        #prefix = os.path.dirname(filename)
+        #if not os.path.exists(prefix):
+        #    os.makedirs(prefix)
+
+        #name = filename or self.filename
+
+        if os.path.exists(filename):
+            with open(filename, 'rb') as dbfile:
+                data = yaml.safe_load(dbfile) or dict()
+                self.data.update(data)
+
     def save(self, filename=None):
         """
         saves the data to the specified filename
@@ -95,8 +137,8 @@ class YamlDB:
         :rtype:
         """
         name = filename or self.filename
-        with open(name, "wb") as stream:
-            yaml.safe_dump(self.data, stream, default_flow_style=False)
+        with open(name, "w") as stream:
+            yaml.dump(self.data, stream, default_flow_style=False)
 
     def flush(self):
         """
@@ -105,13 +147,14 @@ class YamlDB:
         :return:
         :rtype:
         """
-        self.save()
+        if self.backend == ":file:":
+            self.save()
 
     def close(self):
         """
         Close the DB without flushing the current content
         """
-        pass
+        self.flush()
 
     def dict(self):
         """
@@ -164,9 +207,12 @@ class YamlDB:
         :param key: A string representing the value's path in the config.
         :param value: value to be set.
         """
+        try:
+            if value.lower() in ['true', 'false']:
+                value = value.lower() == 'true'
+        except:
+            pass
 
-        if value.lower() in ['true', 'false']:
-            value = value.lower() == 'true'
         try:
             if "." in key:
                 keys = key.split(".")
@@ -192,7 +238,39 @@ class YamlDB:
             print(e)
             raise ValueError("unkown error")
 
-        self.save()
+        self.flush()
+
+    def _delete_keys_from_dict(self, data, keys):
+        # inspired from
+        # https://stackoverflow.com/questions/3405715/elegant-way-to-remove-fields-from-nested-dictionaries
+        for key in keys:
+            with suppress(KeyError):
+                del data[key]
+        for value in data.values():
+            if isinstance(value, MutableMapping):
+                self._delete_keys_from_dict(value, keys)
+
+    def delete(self, item):
+        """
+        Deletes an item form the dict. The key is . separated
+        use it as follows get("a.b.c")
+        :param item:
+        :type item:
+        :return:
+        """
+        try:
+            if "." in item:
+                keys = item.split(".")
+            else:
+                del self.data[item]
+                return
+            self._delete_keys_from_dict(self.data, keys)
+        except Exception as e:
+            print(e)
+            raise ValueError("unkown error")
+
+    def __delitem__(self, key):
+        self.delete(key)
 
     def __getitem__(self, item):
         """
@@ -217,33 +295,6 @@ class YamlDB:
             raise ValueError("unkown error")
         return element
 
-    def __delitem__(self, item):
-        """
-        #
-        # BUG THIS DOES NOT WORK
-        #
-        gets an item form the dict. The key is . separated
-        use it as follows get("a.b.c")
-        :param item:
-        :type item:
-        :return:
-        """
-        try:
-            if "." in item:
-                keys = item.split(".")
-            else:
-                return self.data[item]
-            element = self.data
-            print(keys)
-            for key in keys:
-                element = element[key]
-            del element
-        except KeyError:
-            raise KeyError(f"The key '{item}' could not be found in the yaml file '{self.filename}'")
-        except Exception as e:
-            print(e)
-            raise ValueError("unkown error")
-
     def get(self, key, default=None):
         """
         A helper function for reading values from the config without
@@ -286,5 +337,5 @@ class YamlDB:
         """
         if self.data is None:
             return ""
-        else:
-            return yaml.dump(self.data, default_flow_style=False, indent=2)
+
+        return yaml.dump(self.data, default_flow_style=False, indent=2)
